#include <iostream>
#include <fstream>
#include <iomanip>
#include <map>
#include <string>
#include <sstream>
#include <cctype>
#include <cerrno>

#ifdef GMP
#include <gmpxx.h>
#endif

using namespace std;

#include "parser/SimpleParser.hh"
#include "PseudoBooleanVerifierCallback.h"

int main(int argc, char *argv[])
{
  if (argc!=2)
  {
    cout << "usage: " << argv[0] << " <OPB file>" << endl
	 << "\treads an interpretation on stdin and checks "
	 << "that the opb file is satisfied" << endl
	 << "\tif <OPB file> is a single dash (-), try to read "
	 << "the file name from stdin" << endl;

    exit(1);
  }

  try
  {
    string instance=argv[1];

    PseudoBooleanVerifierCallback<IntegerType> cb;

    ifstream in;

    SimpleParser<PseudoBooleanVerifierCallback<IntegerType> > parser(cb);

    parser.setInput(in);

    // reading the interpretation on stdin

    string s;

    while(cin.good())
    {
      getline(cin,s);

      if (!cin.good()) // deliberately ignore a last line which
		       // doesn't end with '\n'
	break;

      // 
      string keyword="c solving ";
      if(instance=="-" && s.substr(0,keyword.length())==keyword)
      {
	int start,end;
	start=keyword.length();
	while(start<s.length() && isspace(s[start]))
	  ++start;

	end=s.length()-1;
	while(end>=start && isspace(s[end]))
	  --end;

	instance=s.substr(start,start-end+1);
      }

      // we're only concerned by a "v " line
      if (s.length()>=2 && s[0]=='v' && s[1]==' ')
      {
	istringstream f(s);
	char c;
	string word;
	bool positive;
	int idVar;

	f >> word; // skip "v "

	while (f.good())
	{
	  while(isspace(c=f.get()));
	  f.putback(c);

	  // read a literal
	  positive=true;
	  c=f.get();
	  if (c=='-')
	    positive=false;
	  else
	    f.putback(c);

	  c=f.get();

	  if (!f.good())
	    break;

	  if (c!='x')
	  {
	    cerr << "ERROR: variable expected in interpretation ! (read '" 
		 << c << "' instead of 'x')"
		 << endl;
	    exit(7);
	  }
	  
	  f >> idVar;

	  cb.setValue(idVar,positive);
	}
      }
    }

    if (cb.nbAssignedVariables()==0)
    {
      cerr << "ERROR: no interpretation found !" << endl;
      exit(7);
    }

    in.open(instance.c_str());

    // parse the file
    parser.parse();

    cout << "OK\t" << cb.getObjectiveValue() << endl;
    exit(0);
  }
  catch (exception &e)
  {
    cout.flush();
    cerr << "\n\tUnexpected exception :\n";
    cerr << "\t" << e.what() << endl;
    exit(1);
  }
}

